require("kara3d.transform")

-- ********** DEFINITION AND GENERIC FUNCTIONS **********

camera = {class = "camera"}
camera.__index = camera

-- Creates a camera at transform t with horizontal fov f,
-- near plane distance nd, far plane distance fd, and sceen shift (xs, ys)
function camera.new(t, f, xs, ys, nd, fd)
  local xres, yres, _, _ = aegisub.video_size()
  local c = {transform = t, aspect = xres / yres, fov = f or 80 / 180 * math.pi,
    near = nd or 1, far = fd or 1000000000, shiftx = xs or 0, shifty = ys or 0}
  setmetatable(c, camera)
  return c
end

-- Clones this camera
function camera:clone()
  return camera.new(self.transform, self.fov, self.near, self.far)
end

-- ********** INSTANCE FUNCTIONS **********

-- Returns this camera's projection matrix
function camera:projection()
  local sx = 1 / math.atan(self.fov / 2)
  local sy = self.aspect * sx
  local n, f = self.near, self.far
  local m = matrix.new(4, 4,
    {sx, 0,            0,             0,
     0, sy,            0,             0,
     0, 0, (n+f) / (n-f), 2*n*f / (n-f),
     0, 0,            -1,             0})
  return m
end

-- Returns this camera's view matrix
function camera:view()
  return self.transform:inv_matrix(true)
end
